@font-face {
    src: url('fonts/font.eot?#iefix') format('embedded-opentype'),
         url('fonts/font.svg#webfontIyfZbseF') format('svg');
         url('fonts/font.svg#path/to/../../fonts/font.svg') format('svg');
         url('data:font/woff;charset=utf-8;base64,d09GRgABAAAAADJoAA0AAAAAR2QAAQAAAAAAAAAAAAA');
}
div {
    behavior: url("#default#VML");
}
