{% load i18n %}

{% trans "Literal with a percent symbol at the end %" %}
{% trans "Literal with a percent % symbol in the middle" %}

{% trans "It is 100%" %}
{% trans "It is 100%" context "female" %}
{% trans "Looks like a str fmt spec %s but should not be interpreted as such" %}
{% trans "Looks like a str fmt spec % o but should not be interpreted as such" %}

{% trans "1 percent sign %, 2 percent signs %%, 3 percent signs %%%" %}
{% blocktrans with name="Simon" %}{{name}} says: 1 percent sign %, 2 percent signs %%{% endblocktrans %}
